package de.twomartens.troubleshooting

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.constraintlayout.widget.ConstraintLayout.LayoutParams.UNSET
import androidx.core.view.setPadding
import androidx.core.view.updateLayoutParams
import androidx.recyclerview.widget.RecyclerView
import de.twomartens.troubleshooting.databinding.MessageBinding
import java.text.DateFormat

const val VIEW_TYPE_MESSAGE_RECEIVED: Int = 1
const val VIEW_TYPE_MESSAGE_SENT: Int = 2

class MessageListAdapter : RecyclerView.Adapter<MessageListAdapter.MessageHolder>() {
    private var messages: ArrayList<Message> = ArrayList()
    val dateFormat: DateFormat = DateFormat.getTimeInstance(DateFormat.SHORT)

    inner class MessageHolder(private val binding: MessageBinding)
        : RecyclerView.ViewHolder(binding.root) {
        private var message: Message? = null

        fun bind(message: Message) {
            this.message = message
            binding.textMessageBody.text = message.message
            binding.textMessageTime.text = dateFormat.format(message.createdAt)
        }
    }


    // Create new views (invoked by the layout manager)
    override fun onCreateViewHolder(parent: ViewGroup,
                                    viewType: Int): MessageHolder {
        return when (viewType) {
            VIEW_TYPE_MESSAGE_RECEIVED -> {
                val binding = MessageBinding.inflate(LayoutInflater.from(parent.context), parent, false)
                binding.textMessageBody.setBackgroundResource(R.drawable.rounded_rectangle_secondary_color)
                binding.textMessageBody.setTextColor(parent.context.resources.getColor(R.color.color_on_secondary, null))

                binding.textMessageTime.updateLayoutParams<ViewGroup.MarginLayoutParams> {
                    this.marginStart = 4
                    this.marginEnd = UNSET
                }

                binding.textMessageTime.updateLayoutParams<ConstraintLayout.LayoutParams> {
                    this.startToEnd = binding.textMessageBody.id
                    this.endToStart = UNSET
                }

                binding.textMessageBody.updateLayoutParams<ConstraintLayout.LayoutParams> {
                    this.startToStart = parent.id
                    this.topToTop = parent.id
                    this.endToEnd = UNSET
                }

                MessageHolder(binding)
            }
            VIEW_TYPE_MESSAGE_SENT -> {
                val binding = MessageBinding.inflate(LayoutInflater.from(parent.context), parent, false)
                binding.root.setPadding(parent.context.resources.getDimensionPixelSize(R.dimen.message_sent_padding))
                binding.textMessageBody.setBackgroundResource(R.drawable.rounded_rectangle_primary_color)
                binding.textMessageBody.setTextColor(parent.context.resources.getColor(R.color.color_on_primary, null))

                binding.textMessageTime.updateLayoutParams<ViewGroup.MarginLayoutParams> {
                    this.marginEnd = 4
                    this.marginStart = UNSET
                }

                binding.textMessageTime.updateLayoutParams<ConstraintLayout.LayoutParams> {
                    this.endToStart = binding.textMessageBody.id
                    this.startToEnd = UNSET
                }

                binding.textMessageBody.updateLayoutParams<ConstraintLayout.LayoutParams> {
                    this.endToEnd = parent.id
                    this.startToStart = UNSET
                }

                MessageHolder(binding)
            }
            else -> {
                val binding = MessageBinding.inflate(LayoutInflater.from(parent.context), parent, false)
                binding.textMessageBody.setBackgroundResource(R.drawable.rounded_rectangle_secondary_color)
                binding.textMessageBody.setTextColor(parent.context.resources.getColor(R.color.color_on_secondary, null))

                binding.textMessageTime.updateLayoutParams<ViewGroup.MarginLayoutParams> {
                    this.marginStart = 4
                    this.marginEnd = UNSET
                }

                binding.textMessageTime.updateLayoutParams<ConstraintLayout.LayoutParams> {
                    this.startToEnd = binding.textMessageBody.id
                    this.endToStart = UNSET
                }

                binding.textMessageBody.updateLayoutParams<ConstraintLayout.LayoutParams> {
                    this.startToStart = parent.id
                    this.endToEnd = UNSET
                }

                MessageHolder(binding)
            }
        }
    }

    // Replace the contents of a view (invoked by the layout manager)
    override fun onBindViewHolder(holder: MessageHolder, position: Int) {
        holder.bind(messages[position])
    }

    override fun getItemViewType(position: Int): Int {
        val message = messages[position]
        return if (message.sent) {
            VIEW_TYPE_MESSAGE_SENT
        } else {
            VIEW_TYPE_MESSAGE_RECEIVED
        }
    }

    // Return the size of your data set (invoked by the layout manager)
    override fun getItemCount() = messages.size

    fun replaceMessages(messages: ArrayList<Message>) {
        this.messages = messages
        notifyDataSetChanged()
    }
}
