package de.twomartens.troubleshooting

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob

class MessageListViewModel : ViewModel() {
    private val messages: MutableLiveData<ArrayList<Message>> by lazy {
        MutableLiveData<ArrayList<Message>>(ArrayList<Message>())
    }
    val spinnerOptions: MutableLiveData<ArrayList<SpinnerOption>> by lazy {
        MutableLiveData<ArrayList<SpinnerOption>>(ArrayList<SpinnerOption>())
    }
    val solutions: MutableLiveData<ArrayList<CharSequence>> by lazy {
        MutableLiveData<ArrayList<CharSequence>>(ArrayList<CharSequence>())
    }
    val nextSolutionIndex: MutableLiveData<Int> by lazy {
        MutableLiveData<Int>(0)
    }
    val currentSolutionState: MutableLiveData<State> by lazy {
       MutableLiveData<State>(State.HAVE_YOU_TRIED)
    }
    val component: MutableLiveData<Component> by lazy {
        MutableLiveData<Component>()
    }
    val problem: MutableLiveData<Problem> by lazy {
        MutableLiveData<Problem>()
    }
    val initialized: MutableLiveData<Boolean> by lazy { MutableLiveData<Boolean>(false) }
    val spinnerVisible: MutableLiveData<Boolean> by lazy { MutableLiveData<Boolean>(false) }

    private val viewModelJob = SupervisorJob()
    val uiScope = CoroutineScope(Dispatchers.Main + viewModelJob)

    fun getMessages(): LiveData<ArrayList<Message>> = messages

    fun add(message: Message) {
        messages.value?.add(message)
        messages.notifyObserver()
    }

    fun size(): Int {
        val actualMessages: ArrayList<Message>? = messages.value
        return actualMessages?.size ?: 0
    }

    override fun onCleared() {
        super.onCleared()
        viewModelJob.cancel()
    }
}